require 'optparse'
require "deanswerify/version"
require "deanswerify/parser"

module Deanswerify
  # Kicks off the parsing process by parsing command-line arguments and calling
  # the method that parses each input file.
  class Exec
    # @param args [Array<String>] All command-line arguments
    def initialize(args)
      @args = args
      @options = {}
    end

    # Parses the command-line arguments and begins parsing the files.
    #
    # @see #parse
    def parse!
      parse
      exit 0
    end

    # Parses the command-line arguments and begins parsing the files.
    #
    # @see #parse!
    def parse
      @opts = OptionParser.new(&method(:set_opts))
      @opts.parse!(@args)

      process_result
    end

    # Takes the command-line arguments and parses them for optparse.
    #
    # @param opts [OptionParser]
    def set_opts(opts)
      opts.banner = <<EOS
deanswerify v#{Deanswerify::VERSION}

Description:

Parses every file within your current git tree, recursing into
subdirectories and laying waste to any text between the following delimiters:

  I won't be deleted
  /* SOLUTION */
  Text to be deleted
  Multiline is okay
  /* END SOLUTION */
  Case doesn't matter, and you can do /* solution */ inline /* end solution */

Usage (from within a .git repository):

  deanswerify [options]

Options:
EOS

      # Set default options
      @options[:directory] = `git rev-parse --show-toplevel`.delete("\n")
      # opts.on( '-d', '--directory', 'Parse a particular directory' ) do
      # end

      opts.on_tail( '-h', '--help', 'Display this screen' ) do
        puts opts
        exit
      end
    end

    # Processes the input files, removing all answer blocks
    def process_result
      # Get a list of filenames to parse, and ignore the .git folder
      files = %x(find #{@options[:directory]} -type f -not -iwholename '*.git*').split("\n")

      # Parse each file in the filenames array
      Deanswerify::Parser.parse_files files
    end
  end
end
